/* gpregister.c - Handler of the processor registers and bits.
   Copyright 2014	Molnár Károly
*/

#include "stdhdr.h"
#include "libgputils.h"

/*------------------------------------------------------------------------------------------------*/

static int _cdecl _mcu_cmp(const void *P1, const void *P2)
{
  const gp_register_table_t *t1 = (const gp_register_table_t *)P1;
  const gp_register_table_t *t2 = (const gp_register_table_t *)P2;

  return strcasecmp(t1->name, t2->name);
}

/*------------------------------------------------------------------------------------------------*/

static int _cdecl _reg_cmp(const void *P1, const void *P2)
{
  const gp_register_t **r1 = (const gp_register_t **)P1;
  const gp_register_t **r2 = (const gp_register_t **)P2;
  unsigned          a1 = (*r1)->address;
  unsigned          a2 = (*r2)->address;

  if (a1 > a2) {
    return 1;
  }
  else if (a1 < a2) {
    return -1;
  }
  else {
    return 0;
  }
}

/*------------------------------------------------------------------------------------------------*/

static int _cdecl _bit_cmp(const void *P1, const void *P2)
{
  const gp_bit_t *r1 = (const gp_bit_t *)P1;
  const gp_bit_t *r2 = (const gp_bit_t *)P2;
  unsigned    a1 = r1->address;
  unsigned    a2 = r2->address;

  if (a1 > a2) {
    return 1;
  }
  else if (a1 < a2) {
    return -1;
  }
  else {
    return 0;
  }
}

/*------------------------------------------------------------------------------------------------*/

FUNC(const gp_register_table_t*) gp_register_find_mcu(const gp_register_table_t *Table,
	int Table_size, const char *McuName) {
  gp_register_table_t mcu;
  if (!Table || !McuName) return NULL;
  mcu.name = McuName;
  return (const gp_register_table_t *)bsearch(&mcu, Table, Table_size,
	sizeof(gp_register_table_t), _mcu_cmp);
}

/*------------------------------------------------------------------------------------------------*/

const gp_register_t *
gp_register_find_reg(const gp_register_table_t *Mcu, unsigned Address)
{
  gp_register_t         reg;
  const gp_register_t  *rptr = &reg;
  const gp_register_t **ret;

  if (Mcu == NULL) {
    return NULL;
  }

  reg.address = Address;
  ret = (const gp_register_t **)bsearch(&rptr, Mcu->registers, Mcu->register_number,
                                        sizeof(gp_register_t *), _reg_cmp);
  return ((ret) ? *ret : NULL);
}

/*------------------------------------------------------------------------------------------------*/

const char *
gp_register_find_reg_name(const gp_register_table_t *Mcu, unsigned Address)
{
  const gp_register_t *ret;

  ret = gp_register_find_reg(Mcu, Address);
  return ((ret) ? (ret->name) : NULL);
}

/*------------------------------------------------------------------------------------------------*/

const gp_bit_t *
gp_register_find_bit(const gp_register_t *Reg, unsigned Address)
{
  gp_bit_t bit;

  if (Reg == NULL) {
    return NULL;
  }

  bit.address = Address;
  return (const gp_bit_t *)bsearch(&bit, Reg->bits, Reg->bit_number, sizeof(gp_bit_t), _bit_cmp);
}

/*------------------------------------------------------------------------------------------------*/

const char *
gp_register_find_bit_name(const gp_register_t *Reg, unsigned Address)
{
  const gp_bit_t *ret;

  ret = gp_register_find_bit(Reg, Address);
  return ((ret) ? (ret->name) : NULL);
}
